// -----------------------------------------------
// Dynare .mod file to solve:
// IRBC model with two countries (H,F) and quality
// by Diego Gruber and Marta Arespa

// To select quality-(non-)adjusted results go to dyn_calibration.m lines 8-10.
// -----------------------------------------------

write_latex_original_model;

oo_.steady_state=zeros(85,1);

@#define countries = [ "H", "F" ]

// DECLARE MODEL PARAMETERS 

parameters 
  bet $\beta$    // Discounting parameter
  gam $\gamma$    // Returns to quality
  nu  $\nu$    // Elasticity of substitution
  muH $\mu_H$    // Elasticity of labor supply
  muF $\mu_F$    // Elasticity of labor supply
  the $\theta$    // Utility modifier
  FH      // Fixed cost of intermediate firms in units of capital
  FF      // Fixed cost of intermediate firms in units of capital
  tau     // Iceberg transportation costs
  phib $\phi_b$   // Saving costs parameter
  phik $\phi_k$   // Capital adjustment cost parameter
  eta  $\eta$   // Capital adjustment cost parameter
  del  $\delta$   // Capital depreciation rate
  alfH $\alpha_H$   // Home bias parameter at Home
  alfF $\alpha_F$   // Home bias parameter at Foreign
  lam     // GHH parameter
  rho $\rho$    // Cost of q modifier
  aHH     // Shock-process parameters
  aHF
  aFH
  aFF
  psH //to pour these four from pre-computed st st to model equations
  psF
  qsH
  qsF

  ;


// DECLARE EXOGENOUS VARIABLES

varexo  eH eF;

// DECLARE ENDOGENOUS VARIABLES

var r        // World interest rate

  @#for co in countries

    c@{co}   // Consumption
    Y@{co}   // Output of final good
    W@{co}   // "Effective" world income
    q@{co}   // Quality
    p@{co}   // Prices
    d@{co}   // Domestic demand: produced and used domestically
    m@{co}   // Foreign demand: produced abroad, consumed here
    y@{co}   // Output of intermediate good
    l@{co}y  // Manufacturing labor demand
    l@{co}q  // q-enhancing labor demand
    I@{co}   // Number of firms
    Uc@{co}  // Marginal utility of consumption
    n@{co}   // Labor supply
    x@{co}   // Investment
    k@{co}   // Capital
    R@{co}   // Rental price of capital
    b@{co}   // Savings
    w@{co}   // Salaries
    z@{co}   // Technology shocks to manufacturing

    mY@{co}  // Measured Y
    mc@{co}  // Measured c
    mx@{co}  // Measured x
    tot@{co} // Terms of trade
    P@{co}   // Price index
    rer@{co} // Real exchange rate
    ex@{co}  // Exports
    gdp@{co} // GDP
    nx@{co}  // Net exports
    RY@{co}  // Relative output
    RC@{co}  // Relative consumption

    qadj@{co} // Quality adjustment to prices
    qY@{co}   // Quality adjusted Y
    qc@{co}   // Quality adjusted c
    qx@{co}   // Quality adjusted x
    qtot@{co} // Quality adjusted Terms of trade
    qP@{co}   // Quality adjusted Price index
    qrer@{co} // Quality adjusted Real exchange rate
    qex@{co}  // Quality adjusted Exports
    qgdp@{co} // Quality adjusted GDP
    qnx@{co}  // Quality adjusted Net exports
    qRY@{co}  // Quality adjusted Relative output
    qRC@{co}  // Quality adjusted Relative consumption

 @#endfor
  ;



// DECLARE EXTERNAL FUNCTIONS

external_function(name = ppsi,   nargs = 2);
external_function(name = psix,  nargs = 2);
external_function(name = psinv, nargs = 1);
        //external_function(name = pss,   nargs = 1);
        //external_function(name = pss_try,   nargs = 1);
        //external_function(name = qss,   nargs = 1);
        //external_function(name = qss_try,   nargs = 1);
        //external_function(name = Rss,   nargs = 1);
external_function(name = Rss,   nargs = 1);

// CALIBRATION

// Choose case (Default, GHH, Recent)

 @#define case = "Default"
// @#define case = "Heathcoate-Perri"
// @#define case = "Recent"
// @#define case = "GHH"

// Default case
load par.mat //load parameter values used in steadystate_matlab_try.m

set_param_value('bet',bet);
set_param_value('the',the);
set_param_value('phib',phib);
set_param_value('lam',lam);
set_param_value('gam',gam);
set_param_value('tau',tau);
set_param_value('phik',phik);
set_param_value('muH',muH);
set_param_value('muF',muF);
set_param_value('FH', FH);
set_param_value('FF',FF);
set_param_value('del',del);
set_param_value('nu',nu);
set_param_value('eta',eta);
set_param_value('alfH',alfH);
set_param_value('alfF',alfF);
set_param_value('rho',rho);
set_param_value('aHH',aHH);
set_param_value('aFF',aFF);
set_param_value('aFH',aFH);
set_param_value('aHF',aHF);

        

load stst.mat //load steady state values from steadystate_matlab_try.m
    //set_param_value('ps@{co}',pH);
set_param_value('psH',pH);
set_param_value('psF',pF);
    //set_param_value('qs@{co}',qH);
set_param_value('qsH',qH);
set_param_value('qsF',qF);
 


shocks;
  var  eH; stderr 0.00852;//HP(2002)=0.0073//BKK (1994)=0.00852// Gomme and Rupert 2005 (0.0075) //0.013; // match sd(gdpH)
  var  eF; stderr 0.00852;//HP(2002)=0.0044// Gomme and Rupert 2005 (0.0075) //0.013; // "
  corr eH,eF = 0.55;//HP(2002)=0.29//BKK (1994)=0.258 //Islamaj(2014)=[.54,.68] US-ROW estimates// match corr(gdpH,gdpF)      
end;

@#if case == "Heathcoate-Perri"

  del  = 0.03;  // match x/k  
  nu   = 0.665; // match x  
  eta  = 0.95;  // match sd(x)/sd(gdp) 
  alfH = 0.64;  // match m/gdp  
  alfF = alfH;  // "    
  rho  = 0.96;  // 

  aHH  = 0.97;  // 
  aFF  = aHH;   // 
  aFH  = 0.025; // 
  aHF  = aFH;   //  

  shocks;
    var  eH; stderr 0.013;//0.0073; // match sd(gdpH)
    var  eF; stderr  0.013;//0.0073; // "
    corr eH,eF = 0.42;//0.290;     // match corr(gdpH,gdpF)      
  end;

@#endif

@#if case == "GHH"
    
  //muH  = 1.3;//should move btw 1.25 and 1.5 for elast 4 to 2 (literature).   
  //muF  = muH; 
  //lam  = 8;   
  //del  = 0.025;//0.03;
  //eta  = 0.87;//0.955;

shocks;
  var  eH; stderr 0.00852;//HP(2002)=0.0073//BKK (1994)=0.00852// Gomme and Rupert 2005 (0.0075) //0.013; // match sd(gdpH)
  var  eF; stderr 0.00852;//HP(2002)=0.0044// Gomme and Rupert 2005 (0.0075) //0.013; // "
  corr eH,eF = 0.55;//HP(2002)=0.29//BKK (1994)=0.258 //Islamaj(2014)=[.54,.68] US-ROW estimates// match corr(gdpH,gdpF)      
end;

@#endif

@#if case == "Recent"

  alfH = .623;  // match m/gdp  
  alfF = alfH;
  del = 0.028;
  eta  = 0.97;

  shocks;
  var  eH; stderr 0.008; // match sd(gdpH)
  var  eF; stderr 0.008; // "
  corr eH,eF = 0.84;     // match corr(gdpH,gdpF)    
  
end;

@#endif


// MODEL EQUATIONS

model;

  
  @#for co in countries

  @# if co == countries[1]
  @#  define oc = countries[2]
  @# else
  @#  define oc = countries[1]
  @# endif

  
  // 1 Household's budget constraint (c)
     c@{co} = w@{co}*n@{co} + R@{co}*k@{co}(-1) + b@{co}(-1) - phib/2*b@{co}(-1)^2 - x@{co} - r*b@{co};

  // 2 Domestic absorption (Y)
     Y@{co} = c@{co} + x@{co};

  // 3 Adjusted world income (W)
     W@{co} = (alf@{co}^(1/(1-nu))*Y@{co} + (1-alf@{oc})^(1/(1-nu))*tau^(-nu/(1 - nu))*Y@{oc});

  // 4 Intermediate good quality (q)
     q@{co} = ((gam - rho)*z@{co}^(nu/(1 - nu))*W@{co}*(nu/w@{co})^(1/(1 - nu)))^((1 - nu)/(1 - (gam - rho + 1)*nu));
    // different in version 5 : q@{co} = ((gam - rho)*z@{co}^(nu/(1 - nu))*W@{co}*(nu/w@{co})^(1/(1 - nu)))^((1 - nu)/(2-rho - (gam - rho + 1)*nu));//in V5

  // 5 Intermediate good price (p)
     p@{co} = (1/nu)*q@{co}^rho*w@{co}/z@{co};
    // different in version 5 : p@{co} = (1/nu)*q@{co}*w@{co}/z@{co}; //in V5

  // 6 Domestic intermediates demand (d)
     d@{co} = Y@{co}*(alf@{co}*q@{co}^(gam*nu)/p@{co})^(1/(1 - nu));

  // 7 Foreign intermediates demand (m)
     m@{co} = Y@{co}*((1-alf@{co})*q@{oc}^(gam*nu)/(tau*p@{oc}))^(1/(1 - nu));

  // 8 Intermediate goods production (y)
     y@{co} = d@{co} + tau*m@{oc};

  // 9 Manufacturing labor demand (ly)
     l@{co}y = y@{co}*q@{co}^rho/z@{co};
    // different in version 5 : l@{co}y = y@{co}*q@{co}/z@{co};//in V5

  // 10 Quality labor demand (lq)
     l@{co}q = q@{co};
            // different in version 5 :l@{co}q = q@{co}/z@{co}; //in V5

  // 11 Number of firms from Income Accounts identity (I)
     I@{co} = (n@{co}*w@{co}+k@{co}(-1)*R@{co})/(y@{co}*p@{co});

  @#if case=="GHH"

    // 12 Marginal utilities with GHH preferences (Uc)
       Uc@{co} = (c@{co} - lam*n@{co}^mu@{co})^(the-1);

    // 13 FOC for labor (n)
       w@{co} = lam*mu@{co}*n@{co}^(mu@{co}-1);

  @#else

    // 12 Marginal utilities with standard preferences
       Uc@{co} = mu@{co}*c@{co}^(the*mu@{co} - 1)*(1 - n@{co})^(the*(1 - mu@{co}));
            // different in version 5 : Uc@{co} = c@{co}^(the*mu@{co} - 1)*(1 - n@{co})^(the*(1 - mu@{co})); //in V5

    // 13 FOC for labor (n)
       w@{co} = (1 - mu@{co})/mu@{co}*c@{co}/(1 - n@{co}); 
     
  @#endif

  // 14 Capital formation with adjustment costs (k)
     k@{co} = (1-del)*k@{co}(-1) + ppsi(x@{co},k@{co}(-1))*k@{co}(-1);

  // Derivative of x_t wrt K_t
     # dxdk0@{co} = 1/eta*(x@{co}/k@{co}(-1))^(1-eta);

  // Derivative of x_t+1 wrt K_t
     # dxdk1@{co} = -1/eta*(x@{co}(+1)/k@{co})^(1-eta)*k@{co}(+1)/k@{co} + x@{co}(+1)/k@{co};

  // 15 Euler equation for investment (x)
            //Uc@{co} = bet*Uc@{co}(+1)*eta*phik*(k@{co}(-1)/x@{co})^(1-eta)*(R@{co}(+1) + x@{co}(+1)^(1-eta)/(eta*phik)*k@{co}(+1)/k@{co}^(2-eta) - x@{co}(+1)/k@{co});
     Uc@{co}*dxdk0@{co} = bet*Uc@{co}(+1)*(R@{co}(+1) - dxdk1@{co});

  // 16 Euler equation for bond holdings (b)
     Uc@{co} = bet*Uc@{co}(+1)*(1-phib*b@{co})/r;

  // 17 Labor market clearing (w)
     n@{co} = I@{co}*(l@{co}y + l@{co}q);

  // 18 Capital market clearing (R)
     k@{co}(-1) = I@{co}*F@{co};

  // 19 Shock processes (z)
     z@{co} = exp(a@{co}@{co}*log(z@{co}(-1)) + a@{co}@{oc}*log(z@{oc}(-1)) + e@{co});

  // NATIONAL ACCOUNTING

  // 20 Real GDP (gdp)
     gdp@{co} = I@{co}*ps@{co}*(d@{co}+tau*m@{oc}); //d and m* used for c and investment
   

  // 21 Exports (ex)
     ex@{co} = tau*I@{co}*m@{oc}*ps@{co};

  // 22 Measured absorption
     mY@{co} = gdp@{co} - (ex@{co} - ex@{oc});

  // 23 Measured investment
     mx@{co} = x@{co}*mY@{co}/Y@{co};

  // 24 Measured consumption
     mc@{co} = c@{co}*mY@{co}/Y@{co};

  // 25 Terms of trade (tot)
     tot@{co} = (p@{oc}/p@{co})*(ps@{co}/ps@{oc});

  // 26 Price index (P)
     # M@{co} = ex@{oc}/mY@{co};
     P@{co} = (1-M@{co})*p@{co}/ps@{co} + M@{co}*p@{oc}/ps@{oc};

  // 27 Real exchange rate (rer)
     rer@{co} = P@{oc}/P@{co};

  // 28 Net exports (nx)
     nx@{co} = (ex@{co}-ex@{oc})/gdp@{co};
    
  // 29 Relative output (RY)
     RY@{co} = gdp@{co}/gdp@{oc};
    
  // 30 Relative consumption (RC)
     RC@{co} = mc@{co}/mc@{oc};

  // NATIONAL ACCOUNTING WITH QUALITY ADJUSTMENT

  // 31 Quality adjustment factor:
     qadj@{co} = (q@{co}/qs@{co})^rho;

  // 32 Real GDP (gdp)
     qgdp@{co} = gdp@{co}*qadj@{co};

  // 33 Exports (ex)
     qex@{co} = ex@{co}*qadj@{co};

  // 34cMeasured absorption
     qY@{co} = qgdp@{co} - (qex@{co} - qex@{oc});

  // 35 Measured investment
     qx@{co} = x@{co}*qY@{co}/Y@{co};

  // 36 Measured consumption
     qc@{co} = c@{co}*qY@{co}/Y@{co};

  // 37 Terms of trade (tot)
     qtot@{co} = tot@{co}*(qadj@{co}/qadj@{oc});

  // 38 Price index (P)
     # Mq@{co} = qex@{oc}/qY@{co};
     qP@{co} = (1-Mq@{co})*p@{co}/(ps@{co}*qadj@{co}) + Mq@{co}*p@{oc}/(ps@{oc}*qadj@{oc});

  // 39 Real exchange rate (rer)
     qrer@{co} = qP@{oc}/qP@{co};

  // 40 Net exports (nx)
     qnx@{co} = (qex@{co}-qex@{oc})/qgdp@{co};
    
  // 41 Relative output (RY)
     qRY@{co} = qgdp@{co}/qgdp@{oc};
      
  // 42 Relative consumption (RC)
     qRC@{co} = qc@{co}/qc@{oc};

  @#endfor

  // 43 Debt market clearing (r)
     bH = -bF;

end;

// STEADY STATE

resid;
steady(maxit=1000);

  

// SOLVE MODEL

 check;

// Plot IRFs for selected variables
        //stoch_simul(order=1,nomoments,nocorr,noprint,nograph,irf=50)  gdpH mH nxH qH qF wH pH pF totH PH cH PF cF rerH RCH;
stoch_simul(order=1,nomoments,nocorr,noprint,nograph,irf=50)  gdpH qgdpH nxH qnxH qH totH qtotH PH qPH cH qcH rerH qrerH RCH qRCH;

Nsim = 50;

oo_sim = zeros(Nsim,M_.endo_nbr,200);

for i=1:Nsim

    set_dynare_seed(i);
  //set_dynare_seed(60); //80 or 60 cross-c ok
        //NOTE: set_dynare_seed(i) with Nsim=1000 and periods=500 decent results for not adjusted quality version

  stoch_simul(
    order=1
    //,hp_filter=1600
    //,nomoments
    //,nocorr
    ,nograph
    ,noprint
    ,irf=0
    ,periods=200
    );

  oo_sim(i,:,:) = oo_.endo_simul;

end;

//AFTER RUNNING THIS FILE (if any error appears), WRITE IN THE COMMAND WINDOW "model_diagnostics(M_,options_,oo_)" to check for collinearity and other problems

